'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var path = _interopDefault(require('path'));
var Metalsmith = _interopDefault(require('metalsmith'));
var asyncEach = _interopDefault(require('async.each'));
var match = _interopDefault(require('multimatch'));
var isBinaryPath = _interopDefault(require('is-binary-path'));
var cons = _interopDefault(require('consolidate'));

var getEngine = function (name) { return cons[name]; };

function gracefulCopy(src, dest, ref) {
  if ( ref === void 0 ) ref = {};
  var data = ref.data; if ( data === void 0 ) data = {};
  var cwd = ref.cwd; if ( cwd === void 0 ) cwd = process.cwd();
  var clean = ref.clean; if ( clean === void 0 ) clean = true;
  var skipInterpolation = ref.skipInterpolation;
  var engine = ref.engine; if ( engine === void 0 ) engine = 'handlebars';

  return new Promise(function (resolve, reject) {
    var source = path.resolve(process.cwd(), src);
    Metalsmith(source) // eslint-disable-line new-cap
      .source('.')
      .use(template)
      .clean(clean)
      .destination(path.resolve(cwd, dest))
      .build(function (err, files) {
        if (err) { return reject(err) }
        resolve(Object.keys(files));
      });
  })

  function template(files, metalsmith, done) {
    var keys = Object.keys(files);

    var matchedFile;
    if (skipInterpolation) {
      if (typeof skipInterpolation === 'function') {
        matchedFile = skipInterpolation;
      } else {
        var matches = match(keys, skipInterpolation);
        matchedFile = function (file) { return matches.indexOf(file) !== -1; };
      }
    }

    asyncEach(keys, run, done);

    function run(file, done) {
      var content = files[file].contents.toString();

      var shouldSkip = matchedFile && matchedFile(file, content);

      // we skip unmathed files (by multimatch or your own function)
      // and binary files
      if (shouldSkip || isBinaryPath(file)) {
        return done()
      }

      var renderer = getEngine(engine);

      renderer.render(content, data, function (err, res) {
        if (err) {
          return done(err)
        }
        files[file].contents = new Buffer(res);
        done();
      });
    }
  }
}

module.exports = gracefulCopy;
